#############################################################################
##
#W  methods.g             GAP 4 package `browse'                Thomas Breuer
##
#Y  Copyright (C)  2007,  Lehrstuhl D fuer Mathematik,  RWTH Aachen,  Germany
##

#T admit optional argument operations list!

#############################################################################
##
#F  BrowseGapMethods()
##
BindGlobal( "BrowseGapMethods", function()
    local argumentswidth, commentwidth, operations, matrix, methodlist, cats,
          operation, opname, newop, i, methods, newi, j, argnames, comment,
          table;

    argumentswidth:= 25;
    commentwidth:= 30;

    # Get the list of operations, sorted alphabetically.
    operations:= OPERATIONS{ [ 1, 3 .. Length( OPERATIONS ) - 1 ] };;
    SortParallel( List( operations, NameFunction ), operations );

    # Fill the matrix with the information about the methods,
    # ordered by their rank.
    # (Omit default methods for setter and getter operations.)
    matrix:= [];
    methodlist:= [];
    cats:= [];
    for operation in operations do
      opname:= NameFunction( operation );
      newop:= true;
      for i in [ 0 .. GAPInfo.MaxNrArgsMethod ] do
        methods:= METHODS_OPERATION( operation, i );
        newi:= true;
        for j in [ 1, 5+i .. Length( methods ) - 3-i ] do
          argnames:= NAMS_FUNC( methods[ j + i + 1 ] );
          if argnames = fail then
            argnames:= [ "..." ];
          elif i <= Length( argnames ) then
            argnames:= argnames{ [ 1 .. i ] };
          fi;
          comment:= methods[ j + i + 3 ];
          if not ':' in comment then
            comment:= "";
          else
            comment:= comment{ [ Position( comment, ':' ) + 2
                                 .. Length( comment ) ] };
          fi;
          if    Length( opname ) < 7
             or i <> 2
             or not opname{ [ 1 .. 7 ] } in [ "Getter(", "Setter(" ]
             or not comment in [ "system setter",
                                 "default method, does nothing" ] then
            # No formatting of the operation name is necessary,
            # because this column is hidden.
            # (If we would break it into lines then a `CategoryValues'
            # function for column 1 would be needed that removes whitespace
            # from the result of `BrowseData.defaults.work.CategoryValues'.)
            Add( matrix, [
              rec( align:= "tl", rows:= SplitString(
                BrowseData.ReallyFormatParagraph( Concatenation( "( ",
                  JoinStringsWithSeparator( argnames, ", " ),
                  " )" ), argumentswidth, "left" ), "\n" ) ),
              rec( align:= "tl", rows:= SplitString(
                BrowseData.ReallyFormatParagraph( comment, commentwidth,
                  "left" ), "\n" ) ),
              rec( align:= "t", rows:= [ String( methods[ j + i + 2 ] ) ] ),
              ] );
            Add( methodlist, methods[ j + i + 1 ] );
            if newop then
              Add( cats, rec( pos:= 2 * Length( matrix ),
                              level:= 1,
                              value:= opname,
                              separator:= "",
                              isUnderCollapsedCategory:= false,
                              isRejectedCategory:= false ) );
              newop:= false;
            fi;
            if newi then
              Add( cats, rec( pos:= 2 * Length( matrix ),
                              level:= 2,
                              value:= String(i),
                              separator:= "-",
                              isUnderCollapsedCategory:= true,
                              isRejectedCategory:= false ) );
              newi:= false;
            fi;
          fi;
        od;
      od;
    od;

    # Construct and show the browse table.
    NCurses.BrowseGeneric( rec(
      work:= rec(
        align:= "tl",
        header:= t -> BrowseData.HeaderWithRowCounter( t, "GAP Methods",
                          Length( matrix ) ),
        main:= matrix,
        labelsCol:= [ [ rec( rows:= [ "Arguments" ], align:= "l" ),
                        rec( rows:= [ "Comment" ], align:= "l" ),
                        rec( rows:= [ "Rank" ], align:= "r" ),
        ] ],
        sepLabelsCol:= "=",
        sepRow:= "-",
        sepCol:= [ "| ", " | ", " | ", " |" ],
        widthCol:= [ , argumentswidth,, commentwidth ],
        SpecialGrid:= BrowseData.SpecialGridLineDraw,

        Click:= rec(
          select_entry:= rec(
            helplines:= [ "show the chosen method in a pager" ],
            action:= function( t )
              local pos, lines, stream;
              if t.dynamic.selectedEntry <> [ 0, 0 ] then
                pos:= t.dynamic.indexRow[ t.dynamic.selectedEntry[1] ] / 2;
                lines:= "";
                stream:= OutputTextString( lines, true );
                AppendTo( stream, methodlist[ pos ] );
                CloseStream( stream );
                if BrowseData.IsDoneReplay( t.dynamic.replay ) then
                  NCurses.Pager( lines );
                  NCurses.update_panels();
                  NCurses.doupdate();
                  NCurses.curs_set( 0 );
                fi;
              fi;
              return t.dynamic.changed;
            end ),
          select_row:= ~.work.Click.select_entry,
        ),

        sepCategories:= "-",
        startCollapsedCategory:= [ [ NCurses.attrs.BOLD, true, "> " ],
                                   [ NCurses.attrs.BOLD, true, "  > " ],
                                 ],
        startExpandedCategory:= [ [ NCurses.attrs.BOLD, true, "* " ],
                                  [ NCurses.attrs.BOLD, true, "  * " ],
                                 ],
      ),
      dynamic:= rec(
        categories:= [ List( cats, x -> x.pos ), cats, [ 1, 2 ] ],
        isCollapsedRow:= BlistList( [ 1 .. 2*Length( matrix ) + 1 ],
                                    [ 2 .. 2*Length( matrix ) + 1 ] ),
        sortFunctionsForColumns:= [ ,, BrowseData.CompareLenLex ],
      ),
    ) );
end );


#############################################################################
##
##  Add the Browse application to the list shown by `BrowseGapData'.
##
BrowseGapDataAdd( "GAP Operations and Methods", BrowseGapMethods, false, "\
the operations available in the current GAP session, \
together with their methods, \
shown in a browse table \
that is categorized by operations and argument number; \
the columns of the table contain \
the names of the arguments, the comments, and the ranks \
of the methods; \
``click'' shows the code of the method" );


#############################################################################
##
#E

