#############################################################################
##
#W img.gi                                                   Laurent Bartholdi
##
#H   @(#)$Id: img.gi,v 1.21 2008/10/01 13:19:16 gap Exp $
##
#Y Copyright (C) 2006, Laurent Bartholdi
##
#############################################################################
##
##  Iterated monodromy groups
##
#############################################################################

#############################################################################
##
#A AsGroupFRMachine, AsMonoidFRMachine, AsSemigroupFRMachine
#O IMGFRMachine
#A AsIMGFRMachine
##
InstallMethod(AsGroupFRMachine, "(FR) for a group homomorphism",
        [IsGroupHomomorphism],
        function(f)
    if not IsFreeGroup(Source(f)) or Source(f)<>Range(f) then
        return fail;
    fi;
    return FRMachineNC(FRMFamily([1]),Source(f),List(GeneratorsOfGroup(f),x->[x^f]),List(GeneratorsOfGroup(f),x->()));
end);

InstallMethod(AsMonoidFRMachine, "(FR) for a monoid homomorphism",
        [IsMagmaHomomorphism],
        function(f)
    if not IsFreeMonoid(Source(f)) or Source(f)<>Range(f) then
        return fail;
    fi;
    return FRMachineNC(FRMFamily([1]),Source(f),List(GeneratorsOfMonoid(f),x->[x^f]),List(GeneratorsOfMonoid(f),x->()));
end);

InstallMethod(AsSemigroupFRMachine, "(FR) for a semigroup homomorphism",
        [IsMagmaHomomorphism],
        function(f)
    if not IsFreeSemigroup(Source(f)) or Source(f)<>Range(f) then
        return fail;
    fi;
    return FRMachineNC(FRMFamily([1]),Source(f),List(GeneratorsOfSemigroup(f),x->[x^f]),List(GeneratorsOfSemigroup(f),x->()));
end);

InstallMethod(IMGFRMachine, "(FR) for a group FR machine and a word",
        [IsGroupFRMachine,IsAssocWord],
        function(M,w)
    local f, i, out, trans, r, p;
    if FR_LOCAL.IMGISONE(M,w,w,true) then
        M := FRMachineNC(FamilyObj(M),M!.free,M!.transitions,M!.output);
        Setter(IMGRelator)(M,w);
    else
        f := ElementsFamily(FamilyObj(M!.free))!.names;
        if "t" in f then
            i := 1;
            while Concatenation("t",String(i)) in f do i := i+1; od;
            f := FreeGroup(Concatenation(f,[Concatenation("t",String(i))]));
        else
            f := FreeGroup(Concatenation(f,["t"]));
        fi;
        r := RankOfFreeGroup(f);
        i := GroupHomomorphismByImagesNC(M!.free,f,GeneratorsOfGroup(M!.free),GeneratorsOfGroup(f){[1..r-1]});
        p := FR_LOCAL.INVERSE(Output(M,w));
        
        trans := List(M!.transitions,r->List(r,x->x^i));
        Add(trans,List(AlphabetOfFRObject(M),a->Inverse(Transition(M,w,p[a]))^i));
        
        out := ShallowCopy(M!.output);
        Add(out,p);
        
        p := Maximum(List(trans[r],Length));
        p := PositionProperty(trans[r],x->Length(x)=p);
        trans[r][p] := trans[r][p]*w^i*GeneratorsOfGroup(f)[r];
        
        M := FRMachineNC(FamilyObj(M),f,trans,out);
        Setter(IMGRelator)(M,w^i*GeneratorsOfGroup(f)[r]);
    fi;
    return M;
end);

InstallMethod(IMGFRMachine, "(FR) for a group FR machine",
        [IsGroupFRMachine],
        function(M)
    local w;
    if IsIMGFRMachine(M) then
        return M;
    fi;
    for w in Arrangements(GeneratorsOfGroup(M!.free),Length(GeneratorsOfGroup(M!.free))) do
        w := Product(w);
        if FR_LOCAL.IMGISONE(M,w,w,true) then
            M := FRMachineNC(FamilyObj(M),M!.free,M!.transitions,M!.output);
            Setter(IMGRelator)(M,w);
            return M;
        fi;
    od;
    Info(InfoFR,3,"could not find appropriate arrangement of generators giving the IMG relation");
    TryNextMethod(); # will cause an NMF error
end);

InstallMethod(ViewObj, "(FR) for an IMG FR machine",
        [IsIMGFRMachine and IsFRMachineStdRep],
        function(M)
    Print("<FR machine with alphabet ", AlphabetOfFRObject(M), " on ", StateSet(M), "/[ ",IMGRelator(M)," ]>");
end);

InstallMethod(Display, "(FR) for an IMG FR machine",
        [IsIMGFRMachine and IsFRMachineStdRep],
        function(M)
    FR_LOCAL.DISPLAYFRMACHINE(M);
    Print("Relator: ",IMGRelator(M),"\n");
end);


FR_LOCAL.ISTREELIKEPERMUTATIONLIST := function(S,A)
    local s, t;
    S := Concatenation(List(S,x->Cycles(x,A)));
    while Size(S)>1 do
        s := Remove(S);
        t := PositionProperty(S,x->Length(Intersection(x,s))=1);
        if t=fail then return false; fi;
        S[t] := Union(s,S[t]);
    od;
    return Set(S[1])=A;
end;

InstallMethod(IsKneadingMachine, "(FR) for an FR machine",
        [IsGroupFRMachine],
        function(M)
    local S, s, t;

    S := GeneratorsOfFRMachine(M);
    for s in S do
        if not IsOne(FRElement(M,s)) then
            if Sum(List(S,t->Number(AlphabetOfFRObject(M),a->Transition(M,t,a)=s)))>1 then
                return false;
            fi;
        fi;
        for t in Cycles(PermList(Output(M,s)),AlphabetOfFRObject(M)) do
            if Number(t,x->not IsOne(FRElement(M,Transition(M,s,t))))>1 then
                return false;
            fi;
        od;
    od;
    return FR_LOCAL.ISTREELIKEPERMUTATIONLIST(List(S,x->PermList(Output(M,x))),AlphabetOfFRObject(M));
end);

FR_LOCAL.PLANAREMBEDDINGFRMACHINE := function(M,justone)
    local S, aS, result, a, x;
    # could be made faster, see [BS02a: Bruin & Schleicher, Symbolic Dynamics of quadratic polynomials]
    if not IsKneadingMachine(M) then
        return [];
    fi;
    M := AsGroupFRMachine(M);
    S := Correspondence(M);
    if not IsList(S) then
        S := GeneratorsOfFRMachine(M);
    fi;
    aS := Filtered([1..Length(S)],i->not IsOne(FRElement(M,S[i])));
    result := [];
    for a in Arrangements(aS,Length(aS)) do
        x := List([1..Length(aS)],i->Product(S{a{[i+1..Length(aS)]}},S[1]^0)
                  *Product(S{a{[1..i]}}));
        if Transition(M,x[1]^Length(AlphabetOfFRObject(M)),AlphabetOfFRObject(M)[1]) in x then
            if justone then return a; fi;
            Add(result,a);
        fi;
    od;
    return result;
end;

InstallMethod(IsPlanarKneadingMachine, "(FR) for an FR machine",
        [IsGroupFRMachine],
        M->FR_LOCAL.PLANAREMBEDDINGFRMACHINE(M,false)<>[]);
#############################################################################

#############################################################################
##
#M FRElement
##
InstallMethod(FRElement, "(FR) for an IMG FR machine and a word",
        [IsIMGFRMachine, IsAssocWord],
        function(M,init)
    if not init in M!.free then
        Error(init, " must be an element of ", M!.free,"\n");
    fi;
    return Objectify(NewType(FREFamily(M), IsIMGFRElement and IsFRElementStdRep),
                   [M,init]);
end);

InstallMethod(FRElement, "(FR) for a IMG FR machine and an initial list",
        [IsIMGFRMachine, IsList],
        function(M,init)
    return Objectify(NewType(FREFamily(M), IsIMGFRElement and IsFRElementStdRep),
                   [M,M!.pack(init)]);
end);

InstallMethod(ViewObj, "(FR) for an IMG FR element",
        [IsIMGFRElement and IsFRElementStdRep],
        function(E)
    Print("<", Size(AlphabetOfFRObject(E)), "#");
    if IsOne(E![2]) then
        Print("identity ...>");
    else
        Print(E![2], ">");
    fi;
end);

FR_LOCAL.IMGISONE := function(m,relator,w,skip)
    local rws, todo, d, t, seen;
    
    rws := NewFRMachineRWS(m);
    if not IsBound(rws.relator) then
        rws.addgprule(rws.letterrep(relator));
        rws.relator := relator;
    fi;
    seen := NewDictionary([],false);
    todo := NewFIFO([rws.letterrep(w)]);
    for t in todo do
        if skip then
            skip := false;
        else
            t := rws.reduce(rws.cyclicallyreduce(t));
        fi;
        if not IsEmpty(t) then
            if KnowsDictionary(seen,t) then
                return false;
            fi;
            d := rws.pi(t);
            if not FR_LOCAL.ISONE(d[2]) then return false; fi;
            Append(todo,d[1]);
        fi;
    od;
    return true;
end;

InstallMethod(IsOne, "(FR) for an IMG FR element",
        [IsIMGFRElement],
        function(x)
    return FR_LOCAL.IMGISONE(x![1],IMGRelator(x![1]),x![2],false);
end);

InstallMethod(\=, "(FR) for two IMG FR elements",
        IsIdenticalObj,
        [IsIMGFRElement,IsIMGFRElement],
        function(x,y)
    if x![1]<>y![1] then
        Error("Cannot compare IMG FR elements in different machines");
    fi;
    return FR_LOCAL.IMGISONE(x![1],IMGRelator(x![1]),x![2]/y![2],false);
end);

InstallMethod(Order, "(FR) for an IMG FR element",
        [IsIMGFRElement],
        function(e)
        local testing, recur;
    testing := NewDictionary(e,false); # elements we consider
    recur := function(g)
        local d, o, h, ho, i, j;
        if IsOne(g) then
            return 1;
        elif IsGroupFRElement(g) then
            g := FRElement(g,CyclicallyReducedWord(InitialState(g)));
        fi;                 
        if KnowsDictionary(testing,g) then
            return infinity;
        else
            AddDictionary(testing,g);
            d := DecompositionOfFRElement(g);
            o := 1;
            for i in Cycles(PermList(d[2]),AlphabetOfFRObject(g)) do
                h := One(g);
                for j in i do h := h*d[1][j]; od;
                ho := recur(h);
                if ho=infinity then
                    return infinity;
                else
                    o := LcmInt(o,Length(i)*ho);
                fi;
            od;
            return o;
        fi;
    end;
    return recur(e);
end);
#############################################################################

#############################################################################
##
#M PROD
##
InstallMethod(\*, "(FR) for an FR machine and a mapping",
        [IsFRMachine and IsFRMachineStdRep, IsMapping],
        function(M,f)
    local S, N;
    S := StateSet(M);
    if S<>Source(f) or S<>Range(f) then
        Error("\*: source, range and stateset must be the same\n");
    fi;
    N := FRMachineNC(FamilyObj(M),S,List(M!.transitions,r->List(r,x->x^f)),M!.output);
    if FR_LOCAL.IMGISONE(M,IMGRelator(M),IMGRelator(M),true) then
        Setter(IMGRelator)(N,IMGRelator(M));
    else
        Info(InfoFR, 1, "Warning: result of composition is not an IMG machine");
    fi;
    return N;
end);

InstallMethod(\*, "(FR) for a mapping and an FR machine",
        [IsMapping, IsFRMachine and IsFRMachineStdRep],
        function(f,M)
    local S, trans, out, i, pi, x, N;
    S := StateSet(M);
    if S<>Source(f) or S<>Range(f) then
        Error("\*: source, range and stateset must be the same\n");
    fi;
    pi := WreathRecursion(M);
    trans := [];
    out := [];
    for i in [1..Length(M!.output)] do
        x := pi(GeneratorsOfFRMachine(M)[i]^f);
        Add(trans,x[1]);
        Add(out,x[2]);
    od;
    N := FRMachineNC(FamilyObj(M),S,trans,out);
    if FR_LOCAL.IMGISONE(M,IMGRelator(M),IMGRelator(M),true) then
        Setter(IMGRelator)(N,IMGRelator(M));
    else
        Info(InfoFR, 1, "Warning: result of composition is not an IMG machine");
    fi;
    return N;
end);

InstallMethod(\^, "(FR) for a group FR machine and a mapping",
        [IsGroupFRMachine, IsMapping],
        function(M,f)
    local S, trans, out, i, pi, x, finv;
    S := StateSet(M);
    if S<>Source(f) or S<>Range(f) then
        Error("\*: source, range and stateset must be the same\n");
    fi;
    pi := WreathRecursion(M);
    trans := [];
    out := [];
    finv := Inverse(f);
    if finv=fail then return fail; fi;
    for i in [1..Length(M!.output)] do
        x := pi(GeneratorsOfFRMachine(M)[i]^finv);
        Add(trans,List(x[1],x->x^f));
        Add(out,x[2]);
    od;
    x := FRMachineNC(FamilyObj(M),S,trans,out);
    if Tester(IMGRelator)(M) then
        Setter(IMGRelator)(x,IMGRelator(M)^f);
    fi;
    return x;
end);
#############################################################################

#############################################################################
##
#M ChangeFRMachineBasis
##
InstallMethod(ChangeFRMachineBasis, "(FR) for a group FR machine and a list",
        [IsGroupFRMachine, IsCollection],
        function(M,l)
    local trans, i;
    if Length(l)<>Size(AlphabetOfFRObject(M)) or not ForAll(l,x->x in StateSet(M)) then
        Error("Invalid base change ",l,"\n");
    fi;
    trans := [];
    for i in [1..Length(M!.transitions)] do
        Add(trans,List(AlphabetOfFRObject(M),a->l[a]^-1*M!.transitions[i][a]*l[M!.output[i][a]]));
    od;
    i := FRMachineNC(FamilyObj(M),StateSet(M),trans,M!.output);
    if Tester(IMGRelator)(M) then
        Setter(IMGRelator)(i,IMGRelator(M));
    fi;
    return i;
end);

InstallMethod(ChangeFRMachineBasis, "(FR) for an FR machine",
        [IsGroupFRMachine],
        function(M)
    local S, l, s, t, u, v;
    
    S := [];
    for s in GeneratorsOfFRMachine(M) do
        for t in Cycles(PermList(Output(M,s)),AlphabetOfFRObject(M)) do
            if Length(t)>1 then
                Add(S,[s,t]);
            fi;
        od;
    od;
    l := [];
#    l[1] := One(StateSet(M));
    l[Random([1..Length(AlphabetOfFRObject(M))])] := One(StateSet(M));
    while not IsEmpty(S) do
        t := First([1..Length(S)],i->Number(S[i][2],i->IsBound(l[i]))>0);
        if t=fail then
            Error("Action is not transitive");
            return fail;
        fi;
        t := Remove(S,t);
        s := Filtered(t[2],i->IsBound(l[i]));
        if Length(s)>1 then
            Error("Action is not contractible (tree-like)");
            return fail;
        fi;
        s := s[1];
        u := s;
        while true do
            v := PermList(Output(M,t[1],u));
            if v=s then
                break;
            else
                l[v] := LeftQuotient(Transition(M,t[1],u),l[u]);
                u := v;
            fi;
        od;
    od;
    return ChangeFRMachineBasis(M,l);
end);
#############################################################################

#############################################################################
##
#M IMG machine to rational map and back
##
FR_LOCAL.PREC := "undefined";
FR_LOCAL.CLOSE := function(x,y) return (x-y)^2<FR_LOCAL.PREC^2; end;
FR_LOCAL.NORTH_POLE := [MACFLOAT_0,MACFLOAT_0,MACFLOAT_1];
FR_LOCAL.SOUTH_POLE := [MACFLOAT_0,MACFLOAT_0,-MACFLOAT_1];

FR_LOCAL.SP$POINTPLOT := function(p,color,etc)
    if etc="" then etc := "NULL"; fi;
    AppendTo(ValueOption("mapleplot"),Concatenation("pointplot3d(",String(p),",color=",color,",",etc,"),"));
end;

FR_LOCAL.SP$LINEPLOT := function(l,r,g,b,thickness)
    AppendTo(ValueOption("mapleplot"),Concatenation("spacecurve(",String(l),",color=COLOR(RGB,",String(r),",",String(g),",",String(b),"),thickness=",String(thickness),"),"));
end;

FR_LOCAL.SP$MAKENORMAL := function(a,b)
    local j, x, n;
    for j in [b,[1,0,0],[0,1,0],[0,0,1]] do
        x := XProduct(a,j);
        n := Sqrt(x^2);
        if n > FR_LOCAL.PREC then return x/n; fi;
    od;
end;

FR_LOCAL.SP$ORDER := function(spider,connect,makegen)
    # modify spider so that its feet are in CCW order around the body;
    # adjust normal directions so that close normals become equal;
    # return words in spider's group that express connection paths
    # set up the gens accordingly so that product(gens in cyclic order)=1
    local right, up, i, angles, w, g, d;
    
    i := PositionProperty(spider.feet,f->FR_LOCAL.CLOSE(f,spider.body));
    if i=fail then
        right := spider.normal[1]; # use any reference direction
    else
        right := spider.normal[i]; # make sure fixed foot is reference direction
    fi;    
    up := XProduct(spider.body,right);
    angles := List([1..Length(spider.feet)],i->
                   [ATAN2_MACFLOAT(spider.normal[i]*up,spider.normal[i]*right),
                    Sqrt((spider.body-spider.feet[i])^2),
                    "feet",
                    spider.feet[i],
                    spider.normal[i],
                    i]);
    if connect<>fail then
        for i in connect do
            Add(angles,
                [ATAN2_MACFLOAT(i[2]*up,i[2]*right),
                 Sqrt((spider.body-i[1])^2),
                 "connect",
                 i[1],
                 i[2]]);
        od;
    fi;
    for i in angles do
        if i[1]<-FR_LOCAL.PREC then i[1] := MACFLOAT_2PI+i[1]; fi;
    od;
    Sort(angles,function(p,q)
        local d;
        d := AbsoluteValue(p[1]-q[1]);
        if d<FR_LOCAL.PREC or d>MACFLOAT_2PI-FR_LOCAL.PREC then
            return p[2] < q[2];
        else
            return p[1] < q[1];
        fi;
    end);
    for i in [2..Length(angles)] do
        d := AbsoluteValue(angles[i][1]-angles[i-1][1]);
        if d<FR_LOCAL.PREC or d>MACFLOAT_2PI-FR_LOCAL.PREC then
            angles[i][5] := angles[i-1][5]; # force close normals to be equal
        fi;
    od;
    spider.feet := [];
    spider.normal := [];
    if makegen then spider.gen := []; fi;
    connect := [];
    spider.perm := [];
    w := One(spider.group);
    for i in angles do
        if i[3]="feet" then
            Add(spider.feet,i[4]);
            Add(spider.normal,i[5]);
            Add(spider.perm,i[6]);
            g := GeneratorsOfGroup(spider.group)[Length(spider.feet)];
            if makegen then Add(spider.gen,g^w); fi;
            w := g*w;
        else
            Add(connect,[i[4],i[5],w]);
        fi;
    od;
    return connect;
end;
        
FR_LOCAL.SP$INIT := function(pcp,basepoint)
    # create a new spider, with body, feet, gens, group, and normal vectors
    local f, r, i;
    f := FreeGroup(Length(pcp));
    r := rec(body := basepoint,
             feet := pcp,
             group := f,
             normal := []);
    for i in pcp do
        Add(r.normal,FR_LOCAL.SP$MAKENORMAL(basepoint,i));
    od;
    return r;
end;

FR_LOCAL.COLLECTEDREALS := function(list)
    local r, i, j;
    r := [];
    for i in list do
        for j in r do
            if FR_LOCAL.CLOSE(j[1],i) then
                j[2] := j[2]+1;
                i := fail;
                break;
            fi;
        od;
        if i<>fail then Add(r,[i,1]); fi;
    od;
    return r;
end;

FR_LOCAL.SPHEREANGLE := function(base,p,q)
    # compute angle at base between points p and q, measured CCW from p to q.
    # angle in in [0,2Pi]
    local right, up;
    right := SphereXProduct(base,p);
    up := XProduct(base,right);
    q := SphereXProduct(base,q);
    q := ATAN2_MACFLOAT(q*up,q*right);
    if q<0 then return q+MACFLOAT_2PI; else return q; fi;
end;
  
FR_LOCAL.SP$LIFT := function(mother,pre)
    local child, connect, d, i, j, k, step, src, legs,
          output, rank, totallyramified, plotfile,
          LIFTLEG, READLEG;
    
    plotfile := ValueOption("mapleplot");

    LIFTLEG := function(src,normal,step,index)
        # lift feet by rotating using normal vector.
        # Draw CCW semicircle at each critical value.
        # Finish by a circle around the target.
        # Record the intersections with straight legs along the way.
        local n, legs, semilegs, i, j, k, match, center, delta, perp;
        n := Length(src)-1;
        legs := List(src,pre);
        legs[1] := List(connect,x->x[1]);
        i := 2;
        while i<=Length(src) do
            if (src[i-1]-src[i])^2 > 3 then # more than 120 degrees
                Add(src,SphereProject(src[i-1]+XProduct(normal,src[i-1])/3),i);
                Add(legs,pre(src[i]),i);
            else
                match := [];
                for j in [1..d] do
                    k := Filtered([1..d],k->(legs[i][k]-legs[i-1][j])^2<step^2);
                    if Length(k)=0 then # subdivide better
                        Add(src,SphereProject((src[i-1]+src[i])/2),i);
                        Add(legs,pre(src[i]),i);
                        match := fail;
                        break;
                    fi;
                    if i=2 and totallyramified then # use angle at origin
                        delta := FR_LOCAL.SPHEREANGLE(src[1],src[2],connect[1][2]);
                        if delta<FR_LOCAL.PREC then delta := MACFLOAT_2PI; fi;
                        k := List(connect,i->FR_LOCAL.SPHEREANGLE(legs[1][1],legs[2][j],i[2]))-delta/d;
                        k := List(k,x->(x+2*MACFLOAT_2PI) mod MACFLOAT_2PI);
                        Info(InfoFR,4,"For j=",j," found angles ",k);
                        k := Filtered([1..d],i->MACFLOAT_PI-AbsoluteValue(k[i]-MACFLOAT_PI)<1/(10*d));
                        if Length(k)<>1 then
                            Error("Could not guess starting edge -- should not occur");
                        fi;
                    fi;
                    if i=Length(src)+1-n then
                        continue;
                    elif Length(k)=1 then
                        Add(match,k[1]);
                    else
                        Error("Subdivide step -- should not occur");
                        step := step/2;
                        match := fail;
                        break;
                    fi;
                od;
                if match=fail then continue; fi;
                if Length(match)=d and i<>Length(src)+1-n then
                    legs[i] := legs[i]{match};
                    i := i+1;
                else # draw a semicircle
                    center := Remove(src,i);
                    Remove(legs,i);
                    if n=1 then
                        semilegs := ShallowCopy(legs);
                    fi;
                    delta := center-src[i-1];
                    Add(src,SphereProject(center+normal*Sqrt(delta^2)),i);
                    Add(legs,pre(src[i]),i);
                    Add(src,SphereProject(center+delta),i+1);
                    Add(legs,pre(src[i+1]),i+1);
                    if n=1 then
                        Add(src,SphereProject(center-normal*Sqrt(delta^2)),i+2);
                        Add(legs,pre(src[i+2]),i+2);
                        Add(src,src[i-1],i+3);
                        Add(legs,legs[i-1],i+3);
                    fi;
                    n := n-1;
                fi;
            fi;
        od;
        output := List([1..d],j->Position(semilegs[Length(semilegs)],legs[Length(legs)][j]));
        if plotfile<>fail then
            FR_LOCAL.SP$LINEPLOT(src,0,index,1-index,2);
            for i in [1..d] do
                FR_LOCAL.SP$LINEPLOT(List(legs,x->x[i]),1,index,1-index,2);
            od;
        fi;
        Remove(semilegs);
        semilegs := TransposedMat(semilegs);
        legs := TransposedMatMutable(legs);
        if not IsEmpty(semilegs) then
            for i in [1..d] do
                Append(legs[i],Reversed(semilegs[output[i]]));
            od;
        fi;
        return [legs,output];
    end;
    
    READLEG := function(incoming,leg,outgoing,index)
        local w, i, j, j0, tail, head, taildelta, headdelta, dir, skip;
        
        if totallyramified then
            leg := leg{[2..Length(leg)-1]};
            dir := incoming[2];
            i := 1;
            repeat
                dir := XProduct(dir,child.body);
                Add(leg,SphereProject(child.body + step/2 * dir),i);
                i := i+1;
                j := FR_LOCAL.SPHEREANGLE(child.body,leg[i],leg[i-1]);
            until j<=MACFLOAT_PI/2+FR_LOCAL.PREC or j>MACFLOAT_2PI-FR_LOCAL.PREC;
            dir := outgoing[2];
            i := Length(leg)+1;
            repeat
                dir := XProduct(dir,child.body);
                Add(leg,SphereProject(child.body + step/2 * dir),i);
                j := FR_LOCAL.SPHEREANGLE(child.body,leg[i-1],leg[i]);
            until j<=MACFLOAT_PI/2+FR_LOCAL.PREC or j>MACFLOAT_2PI-FR_LOCAL.PREC;
        fi;
        w := incoming[Length(incoming)];
        head := leg[1];
        i := 2;
        while i<=Length(leg) do
            repeat
                tail := head;
                head := leg[i];
                dir := TripleProduct(child.body,tail,head);
                i := i+1;
            until AbsoluteValue(dir)>FR_LOCAL.PREC or i>Length(leg);
            if dir>0 then
                dir := 1;
                j0 := 1;
            else
                dir := -1;
                j0 := rank;
            fi;
            skip := -1;
            for j in [j0,j0+dir..rank+1-j0] do
                if dir*XProduct(tail,head)*child.feet[j] > 0 then
                    continue; # leg j does not reach great circle head-tail
                fi;
                taildelta := child.normal[j]*tail;
                if taildelta*dir > skip*FR_LOCAL.PREC then
                    continue; # tail is on wrong side of leg j
                fi;
                repeat
                    headdelta := child.normal[j]*head;
                    if i>Length(leg) or AbsoluteValue(headdelta)>FR_LOCAL.PREC then
                        break;
                    fi;
                    Info(InfoFR,4,"= at ",index,":",leg[i]);
                    skip := 1; # now we allow tail to be on leg j,
                               # and not strictly on its right
                    tail := head;
                    head := leg[i];
                    i := i+1;
                until false;
                if headdelta*dir>FR_LOCAL.PREC then
                    Info(InfoFR,4,"x at ",index,":", tail,"-",head," generator ",j," dir ",dir);
                    w := w*child.gen[j]^dir;
                elif skip=1 then
                    # back up to segment exiting leg j
                    head := tail;
                    i := i-1;
                    break;
                fi;
            od;
        od;
        w := w/outgoing[Length(outgoing)];
        if false then
            j := Product(GeneratorsOfGroup(child.group)); j := [j,j^-1];
            i := 1; while i <= Length(w)-rank+1 do
                if Subword(w,i,i+rank-1) in j then
                    w := Subword(w,1,i-1)*Subword(w,i+rank,Length(w));
                fi;
                i := i+1;
            od;
        fi;
        return w;
    end;
    
    connect := pre(mother.body);
    d := Length(connect);
    i := Number(connect,x->FR_LOCAL.CLOSE(x,mother.body));
    if i>0 then
        if i<>d then
            Error("Basepoint must be non-ramified and disjoint from its preimages, or totally ramified");
        fi;
        totallyramified := true;
        i := SphereXProduct(mother.normal[1],mother.body)/5;
        repeat
            i := SphereProject((mother.body+i)/2);
            j := pre(i);
        until ForAll(j,x->FR_LOCAL.CLOSE(x,mother.body));
        connect := [];
        for j in j do
            k := SphereXProduct(mother.body,j);
            Add(connect,[mother.body,SphereProject(k+10*FR_LOCAL.PREC*XProduct(k,mother.body)),i,j]);
        od;
    else
        totallyramified := false;
        for i in [1..d] do
            connect[i] := [connect[i],FR_LOCAL.SP$MAKENORMAL(mother.body,connect[i])];
        od;
    fi;
    
    j := [];
    for i in mother.feet do
        Add(j,FR_LOCAL.COLLECTEDREALS(pre(i)));
    od;
    child := FR_LOCAL.SP$INIT(Concatenation(List(j,x->List(x,x->x[1]))),mother.body);
    connect := FR_LOCAL.SP$ORDER(child,connect,true);
    
    j := child.feet;
    if not totallyramified then j := Concatenation(j,[child.body]); fi;
    step := Sqrt(Minimum(List(Combinations(j,2),x->(x[1]-x[2])^2)));
    if step < 10*FR_LOCAL.PREC then
        Error("Children's feet seem too close");
    fi;
    Info(InfoFR,2, "Tracing paths with step ", step);
    
    rank := Length(child.feet);
    child.map := [];
    for i in [1..Length(mother.feet)] do
        if FR_LOCAL.CLOSE(mother.feet[i],mother.body) then
            # adding machine, special handling
            src := List([1..d],j->One(child.group));
            src[1] := GeneratorsOfGroup(child.group)[Position(child.feet,mother.body)];
            Add(child.map,[src,List([1..d],i->1+(i-2) mod d)]);
        else
            src := [mother.body];
            for j in [1..i] do
                if FR_LOCAL.CLOSE(mother.normal[i],mother.normal[j]) and mother.feet[j]<>mother.body then
                    Add(src,mother.feet[j]);
                fi;
            od;
            src := LIFTLEG(src,mother.normal[i],step/3,(i-1)/(Length(mother.feet)-1));
            Add(child.map, [List([1..d],j->READLEG(connect[j],src[1][j],connect[src[2][j]],i)),src[2]]);
        fi;
    od;
    if plotfile<>fail then
        FR_LOCAL.SP$POINTPLOT(mother.feet,"BLUE","style=POINT,symbol=CROSS,symbolsize=20");
        FR_LOCAL.SP$POINTPLOT(mother.body,"CYAN","style=POINT,symbol=CIRCLE,symbolsize=20");
        for i in connect do
            FR_LOCAL.SP$LINEPLOT([mother.body,i[1]],1/2,1/2,1/2,2);
        od;
    fi;
    return child;
end;

FR_LOCAL.SP$DRAWJULIA := function(cachesize,f)
    local i, j, m, n, p, q, prejulia, julia, cache, critexp, deg, df, fp,
          JULIAITER;
    
    deg := DegreeOfRationalFunction(f);
    df := Derivative(f);
    fp := ComplexRootsOfUnivariatePolynomial(NumeratorOfRationalFunction(f)
                  -IndeterminateOfUnivariateRationalFunction(f)*
                  DenominatorOfRationalFunction(f)); # fixed points
    SortParallel(List(fp,x->Norm(Value(Derivative(f),x))),fp);
    fp := fp[Length(fp)]; # the one with largest derivative
    
    JULIAITER := function(p,d)
        local i, j, k;
        Add(julia,p);
        if p=P1infinity then
            k := Complex(0);
            i := 2;
        elif Norm(Complex(p))>1 then
            k := 1/Complex(p);
            i := 2;
        else
            k := Complex(p);
            i := 1;
        fi;
        j := Int(1+(cachesize-FR_LOCAL.PREC)*(RealPart(k)+1)/2);
        k := Int(1+(cachesize-FR_LOCAL.PREC)*(ImaginaryPart(k)+1)/2);
        if cache[i][j][k] then
            return;
        fi;
#        Add(julia,p);
        cache[i][j][k] := true;
        if d<0 then
            for p in P1PreImages(f,p) do
                JULIAITER(p,d+LOG_MACFLOAT(AbsoluteValue(Value(df,Complex(p))))-critexp);
            od;
        fi;
    end;
    
    cache := List([1..2],i->List([1..cachesize],j->BlistList([1..cachesize],[])));
    julia := [];
    prejulia := [[P1Point(fp),1]];
    n := LogInt(1000,deg); # compute at most 1000 points to estimate hausdorff dimension
    for i in [1..n] do
        j := prejulia;
        prejulia := [];
        for p in j do
            for q in P1PreImages(f,p[1]) do
                Add(prejulia,[q,p[2]*Value(df,Complex(q))]);
            od;
        od;
    od;
    for i in prejulia do
        i[2] := LOG_MACFLOAT(AbsoluteValue(i[2]));
        if i[2]<0 then
            Info(InfoFR,2,"Doesn't seem possible to compute critical exponent");
            critexp := MacFloat(0);
            break;
        fi;
    od;
    if not IsBound(critexp) then
        p := MacFloat(0); q := MacFloat(10^8);
        while AbsoluteValue(p-q)>q/100 do
            m := (p+q)/2;
            i := Sum(prejulia,x->EXP_MACFLOAT(-m*x[2]));
            if i>MacFloat(1) then
                p := m;
            else
                q := m;
            fi;
        od;
        critexp := q*LOG_MACFLOAT(MacFloat(deg));
    fi;
    Info(InfoFR,2,"Critical exponent approximated by ",critexp);
    for i in prejulia do
        JULIAITER(i[1],i[2]-n*critexp);
    od;
    return julia;
end;

InstallMethod(IMGFRMachine, "(FR) for a rational function",
        [IsRationalFunction],
        function(f)
    local mother, child, cp, pcp, hom, plotfile, deg, i, j;
    
    if ValueOption("precision")<>fail then
        FR_LOCAL.PREC := ValueOption("precision");
    else
        FR_LOCAL.PREC := MacFloat(10^-5);
    fi;
    
    i := NumeratorOfRationalFunction(f);
    j := DenominatorOfRationalFunction(f);
    cp := List(ComplexRootsOfUnivariatePolynomial(Derivative(i)*j-Derivative(j)*i),P1Point);
    deg := DegreeOfRationalFunction(f);
    while Length(cp)<2*deg-2 do Add(cp,P1Point(infinity)); od;
    cp := List(cp,x->[x,2]);
    i := 1;
    while i<=Length(cp) do
        j := i+1;
        while j<= Length(cp) do
            if P1Distance(cp[i][1],cp[j][1])<FR_LOCAL.PREC then
                Remove(cp,j);
                cp[i][2] := cp[i][2]+1;
            else
                j := j+1;
            fi;
        od;
        i := i+1;
    od;
    Info(InfoFR,2,"Critical points at ",cp);

    pcp := [];
    for i in cp do
        i := i[1];
        repeat
            i := Value(f,i);
            if ForAll(pcp,j->P1Distance(i,j)>FR_LOCAL.PREC) then
                j := First(cp,x->P1Distance(x[1],i)<FR_LOCAL.PREC);
                if j=fail then
                    Add(pcp,i);
                else
                    Add(pcp,j[1]);
                fi;
            else
                break;
            fi;
        until false;
    od;
    Info(InfoFR,2,"Post-critical points at ",pcp);
    
    i := ValueOption("basepoint");
    if i=fail then
        i := FR_LOCAL.NORTH_POLE;
    else
        i := SphereP1(i);
    fi;    
    mother := FR_LOCAL.SP$INIT(List(pcp,SphereP1),i);
    FR_LOCAL.SP$ORDER(mother,fail,false);
    
    plotfile := ValueOption("mapleplot");
    if plotfile<>fail then
        PrintTo(plotfile,"with(plots):display3d(");
        i := ValueOption("julia");
        if i<>fail then
            if not IsInt(i) then # default grid size
                i := 100;
            fi;
            i := List(FR_LOCAL.SP$DRAWJULIA(i,f),SphereP1);
            Info(InfoFR,2,"Computed Julia set with ",Length(i)," points");
            AppendTo(plotfile,Concatenation("pointplot3d(",String(i),",style=POINT,color=BLACK),"));
            FR_LOCAL.SP$POINTPLOT(List(cp,x->SphereP1(x[1])),"red","style=POINT,symbol=DIAMOND,symbolsize=20");
        fi;
    fi;
    child := FR_LOCAL.SP$LIFT(mother,p->List(P1PreImages(f,P1Sphere(p)),SphereP1));
    if plotfile<>fail then
        AppendTo(plotfile,"labels=[x,y,z],axes=normal);\n");
    fi;
    Info(InfoFR,2,"Lifted the spider");
    
    hom := [];
    for i in child.feet do
        j := PositionProperty(mother.feet,f->FR_LOCAL.CLOSE(i,f));
        if j=fail then
            Add(hom,One(mother.group));
        else
            Add(hom,GeneratorsOfGroup(mother.group)[j]);
        fi;
    od;
    hom := GroupHomomorphismByImagesNC(child.group,mother.group,GeneratorsOfGroup(child.group),hom);
    i := FRMachine(mother.group,List(child.map,x->List(x[1],w->w^hom)),List(child.map,x->x[2]));
    Setter(IMGRelator)(i,Product(Reversed(GeneratorsOfGroup(mother.group))));
    SetSpider(i,mother);
    return i;
end);

FR_LOCAL.SP$DTHROOT := function(spider,d)
    # lift spider through z^d
    local child, feet, south, i, j, zeta, z, rank, g;
    
    zeta := EXP_COMPLEX(COMPLEX_2IPI/d);
    rank := Length(spider.feet);
    feet := [FR_LOCAL.NORTH_POLE,FR_LOCAL.SOUTH_POLE];
    for i in [2..rank] do
        if FR_LOCAL.CLOSE(spider.feet[i],FR_LOCAL.SOUTH_POLE) then
            south := i;
        else
            z := Complex(P1Sphere(spider.feet[i]))^(1/d);
            for j in [0..d-1] do
                Add(feet,SphereP1(P1Point(z)));
                z := z*zeta;
            od;
        fi;
    od;
    if not IsBound(south) or not FR_LOCAL.CLOSE(spider.feet[1],FR_LOCAL.NORTH_POLE) then
        Error("I expected the first foot to be infinity and another to be 0");
    fi;
    child := FR_LOCAL.SP$INIT(feet,spider.body);
    FR_LOCAL.SP$ORDER(child,fail,false);

    g := GeneratorsOfGroup(child.group);
    i := List([1..d],j->One(child.group));
    i[1] := g[1];
    child.map := [[i,List([1..d],i->1+(i-2) mod d)]];
    for i in [2..south-1] do
        Add(child.map,[g{Concatenation([i],i+1+(rank-2)*[1..d-1])},[1..d]]);
    od;
    j := [];
    for i in [0..d-2] do
        Add(j,Product(List(g{1+((rank-2)*i+south+[0..rank-3])},Inverse)));
    od;
    Add(j,Product(g{south+[(d-1)*(rank-2),(d-1)*(rank-2)-1..0]}));
    Add(child.map,[j,List([1..d],i->1+i mod d)]);
    for i in [south+1..rank] do
        Add(child.map,[g{i+(rank-2)*[0..d-1]},[1..d]]);
    od;
    return child;
end;

FR_LOCAL.SP$MOEBIUS := function(spider,m)
    # m can be a moebius transformation (see P1Map)
    local p, pinv, g, i, j, x, map, normal, delta, adder, turn;
    normal := [];
    for i in [1..Length(spider.feet)] do
        p := XProduct(spider.normal[i],spider.body);
        p := List([0..2],j->SphereP1(P1Sphere(SphereProject((2-j)*spider.body+j*p))^m));
        normal[i] := SphereXProduct(p[2]-p[1],p[3]-p[2]);
        spider.feet[i] := SphereP1(P1Sphere(spider.feet[i])^m);
    od;
    spider.body := SphereP1(P1Sphere(spider.body)^m);
    spider.normal := [];
    for i in [1..Length(spider.feet)] do
        spider.normal[i] := FR_LOCAL.SP$MAKENORMAL(spider.body,spider.feet[i]);
    od;
    FR_LOCAL.SP$ORDER(spider,fail,false);
    p := spider.perm;
    pinv := FR_LOCAL.INVERSE(p);
    g := GeneratorsOfGroup(spider.group);
    map := [];
    if spider.feet[1]<>spider.body then
        adder := fail;
    else
        adder := FR_LOCAL.SPHEREANGLE(spider.body,spider.normal[1],normal[1]);
    fi;
    for i in [1..Length(p)] do
        delta := normal[p[i]]*spider.body;
        x := g[p[i]];
        turn := FR_LOCAL.SPHEREANGLE(spider.body,spider.normal[i],normal[p[i]]);
        if turn > MACFLOAT_PI then
            turn := turn - MACFLOAT_2PI;
        fi;
        if adder<>fail and i=1 then
            ;
        elif AbsoluteValue(turn)<FR_LOCAL.PREC then
            ;
        elif turn < MACFLOAT_0 then
            for j in Concatenation([p[i]+1..Length(p)],[1..p[i]-1]) do
                if adder<>fail and pinv[j]=1 then
                    ;
                elif normal[j]=normal[p[i]] and j<p[i] then
                    continue;
                elif spider.normal[pinv[j]]=spider.normal[i] and pinv[j]<i then
                    ;
                elif spider.feet[pinv[j]]*spider.normal[i]>MACFLOAT_0 then
                    continue;
                elif spider.feet[pinv[j]]*normal[p[i]]<delta then
                    continue;
                fi;
                x := x^Inverse(g[j]);
            od;
            if adder<>fail then x := x^g[1]; fi;
        else
            for j in Concatenation([p[i]-1,p[i]-2..1],[Length(p),Length(p)-1..1+p[i]]) do
                if adder<>fail and pinv[j]=1 then
                    ;
                elif normal[j]=normal[p[i]] and j<p[i] then
                    ;
                elif spider.normal[pinv[j]]=spider.normal[i] and pinv[j]<i then
                    continue;
                elif spider.feet[pinv[j]]*spider.normal[i]<MACFLOAT_0 then
                    continue;
                elif spider.feet[pinv[j]]*normal[p[i]]>delta then
                    continue;
                fi;
                x := x^g[j];
            od;
            if adder<>fail then x := x^Inverse(g[1]); fi;
        fi;
        if adder<>fail then
            turn := turn+FR_LOCAL.SPHEREANGLE(spider.body,spider.normal[1],spider.normal[i])
                    -(FR_LOCAL.SPHEREANGLE(spider.body,normal[1],normal[p[i]])+adder);
            x := x^(g[1]^Int(RINT_MACFLOAT(turn/MACFLOAT_2PI)));
        fi;
        map[i] := x;
    od;
    if IsBound(spider.twist) then
        spider.twist := GroupHomomorphismByImagesNC(spider.group,Range(spider.twist),g,List(map,x->x^spider.twist));
    else
        spider.perm := map;
    fi;
end;

FR_LOCAL.SP$CHANGEBASEPOINT := function(spider,b)
    local p, pinv, body, normal, g, i, j, map, x, 
          oldconnect, newconnect, oldadder, newadder, oldside, newside;

    if b=spider.body or FR_LOCAL.CLOSE(b,spider.body) then
        return;
    fi;
    body := spider.body; spider.body := b;
    normal := spider.normal; spider.normal := [];
    for i in [1..Length(spider.feet)] do
        spider.normal[i] := FR_LOCAL.SP$MAKENORMAL(spider.body,spider.feet[i]);
    od;
    g := GeneratorsOfGroup(spider.group);
    newconnect := FR_LOCAL.SP$ORDER(spider,[[body,FR_LOCAL.SP$MAKENORMAL(spider.body,body)]],false)[1][2];
    oldconnect := FR_LOCAL.SP$MAKENORMAL(body,spider.body);
    p := spider.perm;
    pinv := FR_LOCAL.INVERSE(p);
    oldadder := Position(spider.feet,body);
    if oldadder=fail then
        oldside := FR_LOCAL.SPHEREANGLE(body,oldconnect,normal[1]);
    else
        oldside := FR_LOCAL.SPHEREANGLE(body,oldconnect,normal[p[oldadder]]);
    fi;
    oldside := oldside < MACFLOAT_PI+FR_LOCAL.PREC and oldside > FR_LOCAL.PREC;
    newadder := Position(spider.feet,spider.body);
    if newadder=fail then
        newside := FR_LOCAL.SPHEREANGLE(spider.body,newconnect,spider.normal[1]);
    else
        newside := FR_LOCAL.SPHEREANGLE(spider.body,newconnect,spider.normal[newadder]);
    fi;
    newside := newside < MACFLOAT_PI+FR_LOCAL.PREC and newside > FR_LOCAL.PREC;
    map := [];
    for i in [1..Length(p)] do
        x := g[p[i]];
        if newconnect=spider.normal[i] and FR_LOCAL.CLOSE(normal[p[i]],oldconnect) then
            for j in [p[i]-1,p[i]-2..1] do
                if spider.normal[pinv[j]]=newconnect and FR_LOCAL.CLOSE(normal[j],oldconnect) and pinv[j]<>oldadder then
                    x := x^g[j];
                else
                    break;
                fi;
            od;
        elif newconnect=spider.normal[i] then
            if oldadder<>fail and not oldside then
                x := x^Inverse(g[p[oldadder]]);
            fi;
        elif FR_LOCAL.CLOSE(normal[p[i]],oldconnect) then
            for j in [Length(p),Length(p)-1..1] do
                if (spider.normal[pinv[j]]=newconnect and FR_LOCAL.CLOSE(normal[j],oldconnect) and pinv[j]<>oldadder) or (spider.feet[pinv[j]]=spider.body and not newside) then
                    x := x^g[j];
                fi;
            od;
        elif newconnect*spider.feet[i]>=FR_LOCAL.PREC or (AbsoluteValue(newconnect*spider.feet[i])<FR_LOCAL.PREC and normal[p[i]]*spider.body > MACFLOAT_0) then
            for j in Concatenation([p[i]+1..Length(p)],[1..p[i]-1]) do
                if pinv[j]=oldadder then
                    if normal[p[oldadder]]*spider.feet[i]>-FR_LOCAL.PREC or oldside then
                        continue;
                    fi;
                elif pinv[j]=newadder then
                    if spider.normal[newadder]*spider.feet[i]<-FR_LOCAL.PREC or not newside then
                        continue;
                    fi;
                elif spider.normal[pinv[j]]=spider.normal[i] and pinv[j]<i then
                    ;
                elif normal[j]=normal[p[i]] and j<p[i] then
                    continue;
                elif spider.normal[pinv[j]]=newconnect and FR_LOCAL.CLOSE(normal[j],oldconnect) then
                    continue;
                elif spider.feet[pinv[j]]*newconnect<0 then
                    continue;
                elif spider.feet[pinv[j]]*spider.normal[i]>0 then
                    continue;
                elif spider.feet[pinv[j]]*normal[p[i]]<0 then
                    continue;
                fi;
                x := x^Inverse(g[j]);
            od;
        else
            for j in Concatenation([p[i]-1,p[i]-2..1],[Length(p),Length(p)-1..1+p[i]]) do
                if pinv[j]=oldadder then
                    if normal[p[oldadder]]*spider.feet[i]<-FR_LOCAL.PREC or not oldside then
                        continue;
                    fi;
                elif pinv[j]=newadder then
                    if spider.normal[newadder]*spider.feet[i]>-FR_LOCAL.PREC or newside then
                        continue;
                    fi;
                elif spider.normal[pinv[j]]=spider.normal[i] and pinv[j]<i then
                    continue;
                elif normal[j]=normal[p[i]] and j<p[i] then
                    ;
                elif spider.normal[pinv[j]]=newconnect and FR_LOCAL.CLOSE(normal[j],oldconnect) then
                    ;
                elif spider.feet[pinv[j]]*newconnect>0 then
                    continue;
                elif spider.feet[pinv[j]]*spider.normal[i]<0 then
                    continue;
                elif spider.feet[pinv[j]]*normal[p[i]]>0 then
                    continue;
                fi;
                x := x^g[j];
            od;
        fi;
        map[i] := x;
    od;
#    Error("test change basepoint");
    if IsBound(spider.twist) then
        spider.twist := GroupHomomorphismByImagesNC(spider.group,Range(spider.twist),g,List(map,x->x^spider.twist));
    else
        spider.perm := map;
    fi;
end;

FR_LOCAL.SP$UNTWIST := function(spider)
    local idle, g, n, ng, w, wg, changed;
    w := List(GeneratorsOfGroup(spider.group),w->w^spider.twist);
    changed := false;
    repeat
        idle := true;
        n := Sum(w,Length);
        for g in GeneratorsOfMonoid(Range(spider.twist)) do
            wg := List(w,x->x^g);
            ng := Sum(wg,Length);
            if ng<n then
                n := ng;
                w := wg;
                idle := false;
                changed := true;
            fi;
        od;
    until idle;
    if changed then
        spider.twist := GroupHomomorphismByImages(Source(spider.twist),Range(spider.twist),
                                GeneratorsOfGroup(spider.group),w);
    fi;
end;

FR_LOCAL.SP$RECURRENT := function(spider)
    # remove feet that map to trivial elements
    local feet, map, i, g, m, result;
    
    feet := ShallowCopy(spider.feet);
    g := ShallowCopy(GeneratorsOfGroup(spider.group));
    i := 1; map := [];
    while i <= Length(feet) do
        if IsOne(g[i]^spider.twist) then
            Remove(g,i);
            Remove(feet,i);
            Add(map,0);
        else
            Add(map,i);
            i := i+1;
        fi;
    od;
    result := FR_LOCAL.SP$INIT(feet,spider.body);
    for i in [1..Length(map)] do
        if map[i]=0 then
            map[i] := One(result.group);
        else
            map[i] := GeneratorsOfGroup(result.group)[map[i]];
        fi;
    od;
    g := MappingGeneratorsImages(spider.twist);
    m := [[],[]];
    for i in [1..Length(g[1])] do
        if not IsOne(g[2][i]) then
            Add(m[1],MappedWord(g[1][i],GeneratorsOfGroup(spider.group),map));
            Add(m[2],g[2][i]);
        fi;
    od;
    result.twist := GroupHomomorphismByImages(result.group,Range(spider.twist),
                            m[1],m[2]);
#    Error("test recurrent");
    return result;
end;

FR_LOCAL.SP$LIFTTWIST := function(mother,child,M,g0,g1,ginfinity)
    # match the alphabets of child and M, and read the new twist
    # setting it in child.twist
    local bij, ctrans, Mtrans, cpos, Mpos, map, i, j,
          src, dst, result;
    
    map := List(GeneratorsOfGroup(mother.group),x->WreathRecursion(M)(x^mother.twist));
    # match the adding machines
    cpos := 1;
    Mpos := First(AlphabetOfFRObject(M),a->not IsOne(map[ginfinity][1][a]));
    if Mpos=fail then
        Error("need to be smarter at building Mpos");
    fi;
    bij := [Mpos];
    ctrans := []; ctrans[cpos] := One(child.group);
    Mtrans := []; Mtrans[Mpos] := One(StateSet(M));
    for i in [2..Length(AlphabetOfFRObject(M))] do
        j := Position(child.map[ginfinity][2],cpos);
        ctrans[j] := child.map[ginfinity][1][j]*ctrans[cpos];
        cpos := j;
        j := Position(map[ginfinity][2],Mpos);
        Mtrans[j] := map[ginfinity][1][j]*Mtrans[Mpos];
        Mpos := j;
        bij[cpos] := Mpos;
    od;
    if not ForAll(ctrans,IsOne) or not ForAll(Mtrans,IsOne) then Error("test"); fi;
    src := [];
    dst := [];
    for i in [1..Length(child.map)] do
        for j in Cycles(PermList(child.map[i][2]),AlphabetOfFRObject(M)) do
            Add(src,Product(child.map[i][1]{j})^ctrans[j[1]]);
            Add(dst,Product(map[i][1]{bij{j}})^Mtrans[bij[j[1]]]);
        od;
    od;
    child.twist := GroupHomomorphismByImages(child.group,StateSet(M),src,dst);
end;

FR_LOCAL.CLEANP1Sphere := function(p)
    p := P1Sphere(p);
    if IS_COMPLEX(p![1]) then
        p![1]![2] := (p![1]![2]+p![1]![1])-p![1]![1]; # zap a small imaginary part
    fi;
    return p;
end;

InstallMethod(RationalFunction, "(FR) for an IMG machine",
        [IsIMGFRMachine],
        function(M)
    local cv, g, d, rank, g0, g1, ginfinity, i, j, z, m, x, iter,
          spider, grandma, child, MMATCH, SANITYCHECK;
    
    if ValueOption("precision")<>fail then
        FR_LOCAL.PREC := ValueOption("precision");
    else
        FR_LOCAL.PREC := MacFloat(10^-10);
    fi;
    
    MMATCH := function(spider,x)
        # return the generator index in spider that is conjugate to x in M
        local g;
        g := GeneratorsOfGroup(spider.group);
        return First([1..Length(g)],i->IsConjugate(StateSet(M),x,g[i]^spider.twist));
    end;
    
    SANITYCHECK := function(where)
        if not IsConjugate(StateSet(M),Product(Reversed(
                   GeneratorsOfGroup(spider.group)),w->w^spider.twist),
                   IMGRelator(M)) then
            Error("After ",iter," iterations, ",where," doesn't preserve the relator");
        fi;
    end;
    
    g := GeneratorsOfFRMachine(M);
    d := Length(AlphabetOfFRObject(M));
    rank := Length(g);
    cv := Filtered(g,x->not FR_LOCAL.ISONE(Output(M,x)));
    z := Indeterminate(COMPLEX_FIELD);
    if Size(cv)>2 then
        Error("Cannot (yet) handle more than two critical values");
    fi;
    g0 := fail;
    g1 := fail;
    ginfinity := fail;
    for i in cv do
        j := First(g,x->ForAny(AlphabetOfFRObject(M),a->IsConjugate(StateSet(M),Transition(M,x,a),i)));
        if i=j then
            ginfinity := i;
        elif j<>fail and not j in cv then
            g0 := i;
            g1 := j;
        fi;
    od;
    if g0=fail then
        if ginfinity=fail then
            z := z^-d;
        else
            z := z^d;
        fi;
        spider := rec(body := P1infinity,
                      feet := [P1infinity,P1Point(0)],
                      iter := 0,
                      group := FreeGroup(2));
        spider.twist := GroupHomomorphismByImages(spider.group,StateSet(M),GeneratorsOfGroup(spider.group),GeneratorsOfFRMachine(M));
        SetSpider(z,spider);
        return z;
    elif ginfinity=fail then
        ginfinity := First(cv,x->x<>g0);
    fi;
    
    spider := FR_LOCAL.SP$INIT(List([1..rank],i->SphereP1(P1Point(i))),FR_LOCAL.NORTH_POLE);
    FR_LOCAL.SP$ORDER(spider,fail,false);
    i := LetterRepAssocWord(CyclicallyReducedWord(IMGRelator(M)));
    if AsSortedList(i)<>[1..rank] then
        Error("Relator (even cyclically reduced) is not of the right form: ",i);
    fi;
    spider.twist := GroupHomomorphismByImagesNC(spider.group,StateSet(M),Reversed(GeneratorsOfGroup(spider.group)),g{i});
    iter := 0;
    repeat
        iter := iter+1;
        m := P1Map(P1Sphere(spider.feet[MMATCH(spider,g0)]),
                   P1Sphere(spider.feet[MMATCH(spider,g1)]),
                   P1Sphere(spider.feet[MMATCH(spider,ginfinity)]),
                   P1Point(0),P1Point(1),P1infinity);
        FR_LOCAL.SP$MOEBIUS(spider,m);
        SANITYCHECK("MOEBIUS");
        FR_LOCAL.SP$CHANGEBASEPOINT(spider,FR_LOCAL.NORTH_POLE);
        SANITYCHECK("CHANGEBASEPOINT");
        Info(InfoFR,2,"Spider's feet at ",List(spider.feet,FR_LOCAL.CLEANP1Sphere));
        
        x := MACFLOAT_0;
        if IsBound(grandma) then
            for i in [1..rank] do
                x := x+Sqrt((spider.feet[i]-grandma.feet[i])^2);
            od;
            if x < FR_LOCAL.PREC then
                break;
            fi;
            Info(InfoFR,4,"Spider moved ",x, " in last iteration");
        fi;

        x := Filtered(Combinations([1..rank],2),p->FR_LOCAL.CLOSE(spider.feet[p[1]],spider.feet[p[2]]));
        if not IsEmpty(x) then
            x := EquivalenceClasses(EquivalenceRelationByPairs(Domain([1..rank]),x));
            x := Filtered(List(x,Elements),c->Size(c)>1);
            x := FR_LOCAL.GETOBSTRUCTION(M,x);
            if x<>fail then return x; fi;
        fi;
        
        child := FR_LOCAL.SP$DTHROOT(spider,d);
        FR_LOCAL.SP$LIFTTWIST(spider,child,M,
                MMATCH(spider,g0),MMATCH(spider,g1),MMATCH(spider,ginfinity));

        grandma := spider;
        spider := FR_LOCAL.SP$RECURRENT(child);
        SANITYCHECK("RECURRENT");
    until false;
    
    i := P1Point(0)^m;
    if i=P1infinity then
        i := COMPLEX_0;
    else
        i := -COMPLEX_1/Complex(i);
    fi;
    j := P1infinity^m;
    if j=P1infinity then
        j := COMPLEX_0;
    else
        j := -COMPLEX_1/Complex(j);
    fi;
    
    m := ((i*z+COMPLEX_1)/(j*z+COMPLEX_1))^d;
    Unbind(spider.normal);
    Unbind(spider.perm);
    spider.iter := iter;
    spider.body := FR_LOCAL.CLEANP1Sphere(spider.body);
    spider.feet := List(spider.feet,FR_LOCAL.CLEANP1Sphere);
    SetSpider(m,spider);
    return m;
end);

FR_LOCAL.GETOBSTRUCTION := function(M,x)
    # x is a list of sets describing coalesced points
    local G, mat, row, w, i, j, c, d;
    G := GeneratorsOfGroup(StateSet(M));
    w := WreathRecursion(M);
    x := List(x,c->Product(G{c})); # not smart... should take product in right order
    mat := [];
    for i in x do
        d := w(i);
        row := List(x,i->0);
        for i in Cycles(PermList(d[2]),AlphabetOfFRObject(M)) do
            c := Product(d[1]{i});
            if ForAny(G,x->IsConjugate(StateSet(M),x,c)) then
                continue; # peripheral curve
            fi;
            for j in [1..Length(x)] do
                if IsConjugate(StateSet(M),x[j],c) then
                    row[j] := row[j] + 1/Length(i);
                    j := true;
                    break;
                fi;
            od;
            if j <> true then
                return fail;
            fi;
        od;
        Add(mat,row);
    od;
    if IsEmpty(NullspaceMat(mat-IdentityMat(Length(x)))) then # no 1 eigenval
        i := List(x,i->1);
        j := List(x,i->1);
        repeat
            i := i*mat;
            j := j*mat*mat;
            if ForAll(j-i,IsNegRat) then
                return fail; # perron-frobenius eigenval < 1
            fi;
        until ForAll(j-i,IsPosRat);
    fi;
    return rec(machine := M,
               obstruction := x,
               matrix := mat);
end;
        
FR_LOCAL.SPIDERPRECISION := MacFloat(10^-10);

FR_LOCAL.SPIDERMAP :=function(M)
    local G, S, s, t, u, v, p, alpha;
    
    # conjugate by group automorphism (the spider algorithm)

    S := [];
    alpha := [];
    G := GeneratorsOfFRMachine(M);
    for s in G do
        for t in Cycles(PermList(Output(M,s)),AlphabetOfFRObject(M)) do
            u := Product(t,x->Transition(M,s,x));
            if not IsOne(u) then
                v := CyclicallyReducedWord(u);
                p := Position(G,v);
                if p=fail then
                    Error("Some state is not conjugate to a generator");
                    return fail;
                elif IsBound(S[p]) then
                    Error("State ",v," is reached twice");
                    return fail;
                fi;
                S[p] := u;
                alpha[p] := [t,Position(G,s)];
            fi;
        od;
    od;
    if not ForAll([1..Length(G)],i->IsBound(S[i])) then
        Error("Some generator is not reached by states");
        return fail;
    fi;
    t := GroupHomomorphismByImages(StateSet(M),StateSet(M),S,G);
    if t=fail then
        Error("Machine is not recurrent");
        return fail;
    fi;
    return [t,alpha];
end;
#############################################################################

#############################################################################
##
#M PolynomialMealyMachine
#M PolynomialFRMachine
#M PolynomialIMGMachine
##
FR_LOCAL.KM$TRUNC := function(a)
    a := a-Int(a);
    if a<0 then return a+1; else return a; fi;
end;

FR_LOCAL.KM$INPART := function(part,x)
    local i, j;
    for i in [1..Length(part)] do
        for j in part[i] do
            if j[1]<=x and x<j[2] then
                return i;
            fi;
        od;
    od;
end;

FR_LOCAL.KM$SPLITPART := function(part,p,epsilon)
    local ind, i, j, newp;
    
    ind := List(p,x->FR_LOCAL.KM$INPART(part,x-epsilon));
    if Size(Set(ind))<>1 then
        Error("Some parts cross");
    fi;
    ind := part[ind[1]];
    i := 1;
    j := 1;
#    Error(ind,p);
    while j<Length(p) do # split off ind\cap [p[j]..p[j+1]]
#        Error("x");
        while ind[i][2]<=p[j] do
            i := i+1;
        od;
        newp := [];
        if ind[i][1]<p[j] then
            Add(ind,[ind[i][1],p[j]],i);
            i := i+1;
            ind[i][1] := p[j];
        fi;
        j := j+1;
        while Length(ind)>=i and ind[i][2]<=p[j] do
            Add(newp,Remove(ind,i));
        od;
        if Length(ind)>= i and ind[i][1]<p[j] then
            Add(newp,[ind[i][1],p[j]]);
            ind[i][1] := p[j];
        fi;
        Add(part,newp);
    od;
end;

FR_LOCAL.KNEADINGMACHINE := function(d,F,J,mode)
    # mode=1: mealy machine
    # mode=2: FR machine equivalent to mode=1
    # mode=3: IMG machine, with standard adding machine=product of gens
    local C, i, j, part, pcp, f, g, trans, t, out, o, p, q, rank;
    
    C := Concatenation(List(F,x->[x,2]),List(J,x->[x,1]));
    for i in C do
        if IsRat(i[1]) then
            i[1] := Set([1..d],j->FR_LOCAL.KM$TRUNC((i[1]+j)/d));
        else
            i[1] := Set(i[1],FR_LOCAL.KM$TRUNC);
        fi;
    od;
    if Sum(C,x->Length(x[1])-1)<>d-1 then
        Error("F and J describe a map of wrong degree");
    fi;
    if ForAny(C,x->Size(Set(d*x[1],FR_LOCAL.KM$TRUNC))<>1) then
        Error("F and J must be ",d,"-prearguments");
    fi;
    
    pcp := [];
    for i in C do
        i := [FR_LOCAL.KM$TRUNC(d*i[1][1]),i[2]];
        while not i in pcp do
            AddSet(pcp,i);
            i := [FR_LOCAL.KM$TRUNC(d*i[1]),i[2]];
        od;
    od;
    rank := Length(pcp);

    part := [[[0,1]]];
    j := 1/2/Lcm(List(C,x->Lcm(List(x[1],DenominatorRat))));
    for i in C do
        FR_LOCAL.KM$SPLITPART(part,i[1],j*(2-i[2]));
    od;
    part := part{List([0..d-1]/d,x->FR_LOCAL.KM$INPART(part,x))};
    
    if mode=1 then
        g := [1..rank+1];
    elif mode=2 then
        f := FreeGroup(rank);
        g := Concatenation(GeneratorsOfGroup(f),[One(f)]);
    elif mode=3 then
        f := FreeGroup(rank+1);
        g := GeneratorsOfGroup(f);
    fi;
    trans := [];
    out := [];
    for i in pcp do
        t := [];
        o := [];
        for j in [0..d-1] do
            q := (i[1]+j)/d;
            p := First(C,x->i[2]=x[2] and q in x[1]);
            if mode=3 then
                if p<>fail then
                    p := p[1][Position(p[1],q) mod Length(p[1])+1];
                    Add(o,1+(j+d*(p-q)) mod d);
                    if p>q then
                        Add(t,Product(g{Filtered([rank,rank-1..1],i->pcp[i][1]>q and pcp[i][1]<p)},One(f))^-1);
                    else
                        Add(t,Product(g{Filtered([rank,rank-1..1],i->pcp[i][1]>=p and pcp[i][1]<=q)},One(f)));
                    fi;
                else
                    Add(o,j+1);
                    p := Position(pcp,[q,i[2]]);
                    if p<>fail then
                        Add(t,g[p]);
                    else
                        Add(t,One(f));
                    fi;
                fi;
            else
                if p<>fail then
                    p := FR_LOCAL.KM$INPART(part,p[1][Position(p[1],q) mod Length(p[1])+1]);
                else
                    p := FR_LOCAL.KM$INPART(part,q);
                fi;
                o[FR_LOCAL.KM$INPART(part,q)] := p;
            
                p := Position(pcp,[q,i[2]]);
                if p<>fail then
                    p := g[p];
                else
                    p := g[Length(g)];
                fi;
                t[FR_LOCAL.KM$INPART(part,q)] := p;
            fi;
        od;
        Add(trans,t);
        Add(out,PermList(o));
    od;
    if mode=1 then
        Add(trans,List([1..d],i->Length(g)));
        Add(out,());
        f := MealyMachine(trans,out);
        SetCorrespondence(f,List([1,2],i->function(r)
            if [r,i] in pcp then
                return Position(pcp,[r,i]);
            else
                return Length(g);
            fi;
        end));
        SetAddingElement(f,Product(g,i->FRElement(f,i)));
    elif mode=2 then
        f := FRMachine(f,trans,out);
        SetCorrespondence(f,List([1,2],i->function(r)
            if [r,i] in pcp then
                return GeneratorsOfFRMachine(f)[Position(pcp,[r,i])];
            else
                return One(StateSet(f));
            fi;
        end));
        SetAddingElement(f,FRElement(f,Product(g)));
    elif mode=3 then
        t := [g[Length(g)]];
        Append(t,List([1..d-1],i->One(f)));
        Add(trans,t);
        Add(out,PermList(List([1..d],i->1+(i-2) mod d)));
        f := FRMachine(f,trans,out);
        Setter(IMGRelator)(f,Product(Reversed(g)));
        SetAddingElement(f,FRElement(f,g[Length(g)]));
    fi;
    return f;
end;

InstallMethod(PolynomialMealyMachine, "(FR) for a degree, Fatou and Julia preangles",
        [IsPosInt,IsList,IsList],
        function(n,F,J)
    return FR_LOCAL.KNEADINGMACHINE(n,F,J,1);
end);

InstallMethod(PolynomialFRMachine, "(FR) for a degree, Fatou and Julia preangles",
        [IsPosInt,IsList,IsList],
        function(n,F,J)
    return FR_LOCAL.KNEADINGMACHINE(n,F,J,2);
end);

InstallMethod(PolynomialIMGMachine, "(FR) for a degree, Fatou and Julia preangles",
        [IsPosInt,IsList,IsList],
        function(n,F,J)
    return FR_LOCAL.KNEADINGMACHINE(n,F,J,3);
end);

FR_LOCAL.MATING := function(machines,adders)
    local w, i, j, states, gen, sgen, sum, f, c, trans, out;
    if not ForAll([1..2],i->IsLevelTransitive(FRElement(machines[i],adders[i]))) then
        Error("Adding machines are not level transitive");
    fi;
    w := List(machines,m->CyclicallyReducedWord(IMGRelator(m)));
    for i in [1..2] do
        j := PositionWord(w[i],adders[i]);
        if j=fail then
            Error("Cannot find adding machine in ",machines[i]);
        fi;
        w[i] := Subword(w[i],j+1,Length(w[i]))*Subword(w[i],1,j-1);
    od;
    w[2] := Product(Reversed(List([1..Length(w[2])],i->Subword(w[2],i,i))));
    
    states := List(machines,x->x!.free);
    gen := []; c := [];
    for i in [1..2] do
        c[i] := Position(GeneratorsOfGroup(states[i]),adders[i]);
        if c[i]<>fail then
            gen[i] := ShallowCopy(GeneratorsOfGroup(states[i]));
            Remove(gen[i],c[i]);
        else
            Error("Adder must be a generator of machine");
        fi;
    od;
    sgen := List(gen,x->List(x,String));
    if Intersection(sgen[1],sgen[2])<>[] then
        sgen := List([1,2],i->List(sgen[i],s->Concatenation(s,String(i))));
    fi;
    f := FreeGroup(Concatenation(sgen));
    sgen := [GeneratorsOfGroup(f){[1..Length(gen[1])]},
             GeneratorsOfGroup(f){[1..Length(gen[2])]+Length(gen[1])}];
    for i in [1..2] do
        Add(sgen[i],fail,c[i]);
        w[i] := MappedWord(w[i],GeneratorsOfGroup(states[i]),sgen[i]);
        sgen[i][c[i]] := w[i];
    od;
    c := List([1..2],i->GroupHomomorphismByImages(states[i],f,
                 GeneratorsOfGroup(states[i]),List(sgen[i],w->w^(3-2*i))));
    gen[2] := List(gen[2],Inverse);
    trans := [];
    out := [];
    for i in [1..2] do
        for j in gen[i] do
            Add(trans,List(AlphabetOfFRObject(machines[i]),a->Transition(machines[i],j,a)^c[i]));
            Add(out,Output(machines[i],j));
        od;
    od;
    sum := FRMachineNC(FamilyObj(machines[1]),f,trans,out);
    SetCorrespondence(sum,c);
    Setter(IMGRelator)(sum,w[1]*w[2]);
    return sum;
end;

InstallMethod(Mating, "(FR) for two IMG machines and an adding machine in each",
        [IsIMGFRMachine,IsIMGFRMachine,IsFRElement,IsFRElement],
        function(M,M2,z,z2)
    return FR_LOCAL.MATING([M,M2],[z,z2]);
end);

InstallMethod(Mating, "(FR) for two IMG machines with adding machines",
        [IsIMGFRMachine and HasAddingElement,IsIMGFRMachine and HasAddingElement],
        function(M,M2)
    return FR_LOCAL.MATING([M,M2],[InitialState(AddingElement(M)),InitialState(AddingElement(M2))]);
end);
#############################################################################

#############################################################################
##
#F DBRationalIMGGroup
##
FR_LOCAL.IMGDB := function()
    local e, f, z, h, H, Z;
    
    f := function(arg)
        local G;
        G := CallFuncList(FRGroup,arg{[4..Length(arg)]});
        Error("!!!need to actually make it an IMG machine!!!");
        SetName(G,Concatenation("IMG(",arg[2],")"));
        G!.Correspondence := arg[3];
        Add(FR_LOCAL.IMGDB,[arg[1],arg[3],G]);
    end;
    e := function(p)
        local k;
        k := AlgebraicExtension(Rationals,p);
        H := RootOfDefiningPolynomial(k);
        Z := Indeterminate(k,"z");
    end;
    
    z := Indeterminate(Rationals,"z");
    h := Indeterminate(Rationals,"h");
    FR_LOCAL.IMGDB := [];
    
    f([2,1],"z^2",z^2,
      "a=<,a>(1,2)");
    f([2,2],"z^-2",z^-2,
      "a=<,a>(1,2)");
    f([3,1],"(z-1)^2",(z-1)^2,
      "a=<,b>(1,2)","b=<,b*a/b>");
    f([3,2],"(2z-1)^2",(2*z-1)^2,
      "a=(1,2)","b=<a,b>");
    f([3,3],"(2z-1)^-2",(2*z-1)^-2,
      "a=<,a^-1/b>(1,2)","b=<a,b>");
    f([3,4],"((z-1)/(z+1))^2",((z-1)/(z+1))^2,
      "a=<,b>(1,2)","b=<a,a^-1/b>");
    f([3,5],"((z-1)/z)^2",((z-1)/z)^2,
      "a=<,b>(1,2)","b=<,a^-1/b>");
    e(2*h^3+2*h^2+2*h+1);
    f([4,1,1],"(z/h+1)^2|2h^3+2h^2+2h+1=0,h~-0.64",(Z/H+1)^2,
      "a=<,a>(1,2)","b=<,a/c>","c=<b,a/b>");
    f([4,1,2],"((z/h+1)^2|2h^3+2h^2+2h+1=0,h~-0.17-0.86i",(Z/H+1)^2,
      "a=(1,2)","b=<a,>","c=<b,c>");
    f([4,2],"((z/i+1)^2",(z/E(4)+1)^2,
      "a=(1,2)","b=<c,c*a/c>","c=<,c*b/c>");
    e(h^3+h^2+2*h+1);
    f([4,3,1],"(z/h+1)^2|h^3+h^2+2h+1=0,h~-0.56",(Z/H+1)^2,
      "a=<,a>(1,2)","b=<,a/c>","c=<b,a/c>");
    f([4,3,2],"(z/h+1)^2|h^3+h^2+2h+1=0,h~-0.21-1.30i",(Z/H+1)^2,
      "a=<,a*c/a>(1,2)","b=<a,>","c=<a*b/a,>");
    H := (1+Sqrt(-7))/4;
    f([4,4],"((z+h)/((-1-2h)z+h))^2|2h^2-h+1=0",((z+H)/((-1-2*H)*z+H))^2,
      "a=(1,2)","b=<c^-1*b*c,a>","c=<c^-1/b/a,c>");
    f([4,5],"((z-1)/(-2z-1))^2",((z-1)/(-2*z-1))^2,
      "a=<,a*c*b/c/a>(1,2)","b=<,a*c*b*a/b/c/a>","c=<a*c/a,b^-1/c/a>");
    f([4,7,1],"((z+i)/(z-i))^2",((z+E(4))/(z-E(4)))^2,
      "a=(1,2)","b=<c*a*b,b*a*b>","c=<b,b*a*c*a*b>");
    f([4,7,2],"((z+1+sqrt2)/(z-1-sqrt2))^2",((z+1+Sqrt(2))/(z-1-Sqrt(2)))^2,
      "a=(1,2)","b=<a,a*b*c>","c=<b,a*b*c*b*a>");
    f([4,7,3],"((z+1-sqrt2)/(z-1+sqrt2))^2",((z+1-Sqrt(2))/(z-1+Sqrt(2)))^2,
      "a=(1,2)","b=<b*a*c,a>","c=<b,c>");
    e(h^3+4*h^2+6*h+2);
    f([4,8,1],"(hz+1)^-2|h^3+4h^2+6h+2=0,h~-0.45",(H*Z+1)^-2,
      "a=<,a^-1/c/b>(1,2)","b=<a,>","c=<a^-1*b*a,a^-1*c*a>");
    f([4,8,2],"(hz+1)^-2|h^3+4h^2+6h+2=0,h~-1.77+1.11i",(H*Z+1)^-2,
      "a=<,c^-1/a/b>(1,2)","b=<,c^-1/a*c>","c=<b,c>");
    H := (1+Sqrt(-7))/4;
    f([4,9],"((z-h/(1+2h))/(z+h))^2|2h^2-h+1=0",((z-H/(1+2*H))/(z+H))^2,
      "a=(1,2)","b=<a*c*a,c^-1/b/a>","c=<a,a*b*a>");
    f([4,10],"(-z/2+1)^-2",(-z/2+1)^-2,
      "a=<,b^-1>(1,2)","b=<c,b^-1*a>","c=<,b>");
    f([4,11],"((z-1)/(z+1+i))^2",((z-1)/(z+1+E(4)))^2,
      "a=<,c^-1*a*b>(1,2)","b=<b^-1/a,c>","c=<,c^-1*a>");
    e(h^3-h^2+3*h+1);
    f([4,12,1],"((z+h)/(z-h))^2|h^3-h^2+3h+1=0,h~-0.29",((Z+H)/(Z-H))^2,
      "a=<,c>(1,2)","b=<a,c/b/c/a/c>","c=<,c*b/c>");
    f([4,12,2],"((z+h)/(z-h))^2|h^3-h^2+3h+1=0,h~-0.64-1.72i",((Z+H)/(Z-H))^2,
      "a=<,b*a*c/a/b>(1,2)","b=<c^-1/a/b,b*a*c*a/c/a/b>","c=<,b*a*c*b/c/a/b>");
    f([4,13],"((z-1)/(z+1/2+i/2))^2",((z-1)/(z+1/2+E(4)/2))^2,
      "a=<,b>(1,2)","b=<,a^-1/c/b>","c=<a,b*c/b>");
    f([4,14],"((z-1)/(-z/2-1))^2",((z-1)/(-z/2-1))^2,
      "a=<,b>(1,2)","b=<a,b*c/b>","c=<,b/c/b/a/b>");
    f([4,15,1],"((-3/2+sqrt5/2)z+1)^-2",((-3/2+Sqrt(5)/2)*z+1)^-2,
      "a=<,c^-1>(1,2)","b=<,c^-1*a>","c=<,c^-1*b>");
    f([4,15,2],"((-3/2-sqrt5/2)z+1)^-2",((-3/2-Sqrt(5)/2)*z+1)^-2,
      "a=<,a^-1>(1,2)","b=<,c^-1*a>","c=<b,>");
    f([4,16],"((z-1)/(z+1/2+sqrt3i/2))^2",((z-1)/(z+E(6)))^2,
      "a=<,a*b/a>(1,2)","b=<,c^-1/b/a>","c=<a,>");
end;

InstallGlobalFunction(ValueRational, function(f,x)
    local n, d, finv;
    if x=infinity then
        finv := Value(f,1/IndeterminateOfUnivariateRationalFunction(f));
        n := Value(NumeratorOfRationalFunction(finv),0);
        d := Value(DenominatorOfRationalFunction(finv),0);
        if d=0 then return infinity; else return n/d; fi;
    else
        n := Value(NumeratorOfRationalFunction(f),x);
        d := Value(DenominatorOfRationalFunction(f),x);
        if d=0 then return infinity; else return n/d; fi;
    fi;
end);

InstallGlobalFunction(CriticalValuesQuadraticRational, function(f)
    local roots, m, z;
    z := IndeterminateOfUnivariateRationalFunction(f);
    roots := [];
    m := function(func,exp)
        local d, e, r, k;
        d := CoefficientsOfUnivariatePolynomial(NumeratorOfRationalFunction(Derivative(func)));
        if Length(d)=2 then
            r := [-d[1]/d[2]];
        elif Length(d)=3 then
            e := d[2]^2-4*d[3]*d[1];
            if IsRat(e) then
                e := Sqrt(e);
            else
                k := AlgebraicExtension(z^2-e);
                e := RootOfDefiningPolynomial(k);
                z := Indeterminate(k,String(z));
                f := Value(f,z);
            fi;
            r := [(-d[2]-e)/2/d[3],(-d[2]+e)/2/d[3]];
        else return; fi;
        if exp=1 then
            UniteSet(roots,r);
        else
            UniteSet(roots,List(r,x->ValueRational(1/z,x)));
        fi;
    end;
    m(f,1);
    m(1/Value(f,1/z),-1);
    if Length(roots)=0 then
        roots := [0,infinity];
    elif Length(roots)=1 then
        Add(roots,infinity);
    fi;
    return List(roots,x->ValueRational(f,x));
end);

InstallGlobalFunction(CanonicalQuadraticRational, function(f)
    local d, m, z;
    if DegreeOfLaurentPolynomial(NumeratorOfRationalFunction(f))>2 or
       DegreeOfLaurentPolynomial(DenominatorOfRationalFunction(f))>2 then
        return fail;
    fi;
    z := IndeterminateOfUnivariateRationalFunction(f);
    m := [];
    for d in CriticalValuesQuadraticRational(f) do
        if IsInfinity(d) then Append(m,[1,0]); else Append(m,[d,1]); fi;
    od;
    f := Value((m[2]*z-m[1])/(-m[4]*z+m[3]),Value(f,(m[3]*z+m[1])/(m[4]*z+m[2])));
    m := ValueRational(f,infinity);    
    if IsZero(m) or IsInfinity(m) then m := ValueRational(f,0); fi;
    if IsZero(m) then
        return [z^2];
    elif IsInfinity(m) then
        return [z^-2];
    fi;
    return Set([Value(f,z*m)/m,m/Value(f,1/m/z)]);
end);

InstallGlobalFunction(PostCriticalMachine, function(f)
    local states, trans, x, i;
    trans := [];
    states := CriticalValuesQuadraticRational(f);
    i := 1;
    while i <= Length(states) do
        x := ValueRational(f,states[i]);
        if x in states then
            Add(trans,[Position(states,x)]);
        else
            Add(states,x);
            Add(trans,[Length(states)]);
        fi;
        i := i+1;
        if RemInt(i,10)=0 then
            Info(InfoFR, 2, "PostCriticalMachine: at least ",i," states");
        fi;
    od;
    i := MealyMachineNC(FRMFamily([1]),trans,List(trans,x->[1]));
    SetCorrespondence(i,states);
    return i;
end);

InstallGlobalFunction(DBRationalIMGGroup, function(arg)
    local i, f;
    if IsFunction(FR_LOCAL.IMGDB) then FR_LOCAL.IMGDB(); fi;
    if arg=[] then
        return  FR_LOCAL.IMGDB;
    elif IsCollection(arg) and IsInt(arg[1]) then
        for i in FR_LOCAL.IMGDB do
            if arg=i[1] then return i[3]; fi;
        od;
        return fail;
    elif IsUnivariateRationalFunction(arg[1]) then
        f := CanonicalQuadraticRational(arg[1]);
        for i in FR_LOCAL.IMGDB do
            if i[2] in f then return i[3]; fi;
        od;
        return fail;
    else
        Error("Argument should be indices in Dau's table, or a rational function\n");
    fi;
end);
#############################################################################

#E img.gi . . . . . . . . . . . . . . . . . . . . . . . . . . . . . ends here
