##############################################################################
##
#W  gpd.g                       GAP4 package `Gpd'               Chris Wensley
#W                                                                & Emma Moore
##
#H  @(#)Id: "gpd/examples/gpd.g", v.1.05 2008/11/21  
##
#Y  Copyright (C) 2008, Chris Wensley and Emma Moore,  
#Y  School of Computer Science, Bangor University, U.K. 

SetInfoLevel( InfoGpd, 1 );

##  TraceImmediateMethods( false );

Print( "\n===============================================================\n");
Print( "<<<< testing examples in the Gpd manual (version 21/11/08) >>>>\n" );
Print( "<<<<      functions for groupoids and their morphisms      >>>>\n" );
Print( "===============================================================\n\n");

Print( "Some filter ranks:\n" ); 
Print( "RankFilter( IsMagmaWithObjects ) = ", 
        RankFilter( IsMagmaWithObjects ), "\n" ); 
Print( "RankFilter( IsGroupoid ) = ", RankFilter( IsGroupoid ), "\n" ); 
Print( "RankFilter( IsSinglePiece ) = ", 
        RankFilter( IsSinglePiece ), "\n" ); 
Print( "RankFilter( IsSinglePieceRaysRep ) = ", 
        RankFilter( IsSinglePieceRaysRep ), "\n" ); 

######################################################## 

Print( "\nSome permutation groupoids:-\n" ); 
d8 := Group( (1,2,3,4), (1,3) );;
SetName( d8, "d8" );
Gd8 := SinglePieceGroupoid( d8, [-9,-8,-7] );
Print( "Gd8 = ", Gd8, "\n" );

c6 := Group( (5,6,7)(8,9) );;
SetName( c6, "c6" );
Gc6 := DomainWithSingleObject( c6, -6 );
Print( "Gc6 = ", Gc6, "\n" ); 

Gd8c6 := UnionOfPieces( [ Gd8, Gc6 ] ); 
Print( "Gd8c6 = " ); 
Display( Gd8c6 );
SetName( Gd8, "Gd8" );
SetName( Gc6, "Gc6" );
SetName( Gd8c6, "Gd8+Gc6" );

Print( ObjectList( Gd8c6 ), "\n" );

s4 := Group( (1,2,3,4), (3,4) );
SetName( s4, "s4" );
Gs4 := SinglePieceGroupoid( s4, [-19,-18,-17] );
Print( "Gs4 = ", Gs4, "\n" );
Gd8r := SinglePieceGroupoidWithRays( Gs4, d8, [(),(2,3),(2,4)] );
Print( "Gd8r = ", Gd8r, "\n" );

Print( "\nSome fp- or pc-groupoids:-\n" ); 
f2 := FreeGroup( 2 );
SetName( f2, "f2" );
Gf2 := Groupoid( f2, -22 );
q8 := SmallGroup( 8,4 );
Gq8 := Groupoid( q8, [ -28, -27 ] );
SetName( Gq8, "Gq8" );
Gq8f2 := Groupoid( [ Gf2, Gq8 ] );
Print( [ IsFpGroupoid(Gf2), IsPcGroupoid(Gq8), IsPcGroupoid(Gq8f2) ], "\n\n" );

e1 := MultiplicativeElementWithObjects( Gd8, (1,2)(3,4), -9, -8 );
e2 := MultiplicativeElementWithObjects( Gd8, (1,3), -8, -7 );
prod := e1*e2;
Print( "e1 = ", e1, "\ne2 = ", e2, "\n" );
Print( "prod = ", prod, "\n" );
e3 := MultiplicativeElementWithObjects( Gd8, (1,3)(2,4), -7, -9 );
cycle := prod*e3;
Print("e3 = ",e3, "\ncycle = ",cycle, "  has order = ", Order(cycle), "\n\n" );
count := 0; 
for a in Gd8 do 
    count := count+1; 
    if ( ( a![1] = (1,3) ) and ( a![2] = -8) )  then 
        Print( count, " : ", a, "\n" ); 
    fi; 
od;

Print( [ Size( Gd8 ), Size( Gc6 ), Size( Gd8c6 ), Size( Gq8f2 ) ], "\n\n" );

G4 := Groupoid( [ Gd8c6, Gf2, Gq8 ] );
Print( "G4 = " ); 
Display( G4 );
Print( "Size of G4 is ", Size(G4), "\n\n" );
G5 := Groupoid( [ Gq8, Gf2, Gd8c6 ] );
Print( "G4 = G5 ?  ", G4=G5, "\n\n" );

### Section 2.2 : Subgroupoids ###

c4d := Subgroup( d8, [ (1,2,3,4) ] ); 
SetName( c4d, "c4d" );
k4d := Subgroup( d8, [ (1,2)(3,4), (1,3)(2,4) ] );
SetName( k4d, "k4d" );

## problems 18/09/08
L := [ [ k4d, [-9] ], [ c4d, [-8,-7] ] ]; 
compU := List( L, c -> SinglePieceGroupoid( c[1], c[2] ) ); 
U := UnionOfPieces( compU );
Print( "U = ", U, "\n" ); 

Ud8 := Subgroupoid( Gd8, [ [ k4d, [-9] ], [ c4d, [-8,-7] ] ] );
SetName( Ud8, "Ud8" );
Uk4d := Subgroupoid( Gd8, [ [ k4d, [-9] ] ] );
SetName( Uk4d, "Uk4d" );
Uc4d := Subgroupoid( Gd8, [ [ c4d, [-8,-7] ] ] );
SetName( Uc4d, "Uc4d" ); 
Print( "Ud8 = " ); 
Display( Ud8 );
Print( "\n" ); 
