############################################################################# 
## 
#W  mwo.g                     GAP4 package `Gpd'                Chris Wensley 
##
#H  @(#)Id: "gpd/examples/mwo.g" , v.1.05 2008/11/21  
##
#Y  Copyright (C) 2008, Chris Wensley,  
#Y  School of Computer Science, Bangor University, U.K. 
#Y

SetInfoLevel( InfoGpd, 1 );
## TraceImmediateMethods( true );


Print( "\n===============================================================\n");
Print( "<<<< testing examples in the Gpd manual (version 21/11/08) >>>>\n" );
Print( "<<<< functions for magmas with objects and their mappings  >>>>\n" );
Print( "===============================================================\n\n");


Print("\n========================  MAGMA  ============================\n\n");

tm := [[1,2,4,3],[1,2,4,3],[3,4,2,1],[3,4,2,1]]; 
Display( tm );
m := MagmaByMultiplicationTable(tm); 
SetName( m, "m" ); 
m1 := MagmaElement(m,1); 
m2 := MagmaElement(m,2); 
m3 := MagmaElement(m,3); 
m4 := MagmaElement(m,4); 
Print( "One(m) = ", One(m), "\n\n" ); 
M78 := MagmaWithObjects( m, [-8,-7] ); 
Print( "M78 = ", M78, "\n" ); 
SetName( M78, "M78" ); 
Print( "Is M78 associative, commutative? ", 
    [ IsAssociative(M78), IsCommutative(M78) ], "\n\n" ); 
Print( KnownTruePropertiesOfObject( M78 ), "\n\n" ); 

a78 := MultiplicativeElementWithObjects( M78, m2, -7, -8 ); 
Print( "a78 = ", a78, "\n" );
b87 := MultiplicativeElementWithObjects( M78, m3, -8, -7 ); 
Print( "b87 = ", b87, "\n" );
ba := b87*a78; 
ab := a78*b87;
Print( "[ba,ab,a78^2,ba^2] = ", [ba,ab,a78^2,ba^2], "\n" );


Print("\n\n====================  SEMIGROUP  ==========================\n\n");

t := Transformation( [1,1,2,3] ); 
u := Transformation( [2,3,4,4] ); 
sgp := Semigroup( t, u ); 
SetName( sgp, "sgp<t,u>" ); 
S123 := SemigroupWithObjects( sgp, [-3,-2,-1] ); 
Print( "S123 = ", S123, "\n" ); 
Print( "Is S123 associative, commutative? ", 
    [ IsAssociative(S123), IsCommutative(S123) ], "\n\n" ); 
Print( KnownTruePropertiesOfObject( S123 ), "\n\n" ); 
Print( CategoriesOfObject( S123 ), "\n\n" ); 

t12 := MultiplicativeElementWithObjects( S123, t, -1, -2 ); 
u23 := MultiplicativeElementWithObjects( S123, u, -2, -3 ); 
tu31 := MultiplicativeElementWithObjects( S123, t*u, -3, -1 ); 
Print( "[t12,u23,tu31] = ", [t12,u23,tu31], "\n" );
Print( "[t12*u23,u23*tu31,tu31*t12] = ", [t12*u23,u23*tu31,tu31*t12], "\n" );


Print("\n=========================  MONOID  =========================\n\n");
v := Transformation( [2,3,1] ); 
w := Transformation( [2,2,3] ); 
mon := Monoid( v, w ); 
SetName( mon, "mon<v,w>" ); 
M456 := MonoidWithObjects( mon, [-6,-5,-4] ); 
Print( "M456 = ", M456, "\n" ); 
Print( "Is M456 associative, commutative? ", 
    [ IsAssociative(M456), IsCommutative(M456) ], "\n\n" ); 
Print( KnownTruePropertiesOfObject( M456 ), "\n\n" ); 
Print( CategoriesOfObject( M456 ), "\n\n" ); 

v45 := MultiplicativeElementWithObjects( M456, v, -4, -5 ); 
w56 := MultiplicativeElementWithObjects( M456, w, -5, -6 ); 
vw64 := MultiplicativeElementWithObjects( M456, v*w, -6, -4 ); 
Print( "[v45,w56,vw64] = ", [v45,w56,vw64], "\n" );
Print( "[v45*w56,w56*vw64,vw64*v45] = ", [v45*w56,w56*vw64,vw64*v45], "\n" );


Print("\n================ GROUP AS SINGLE OBJECT GROUPOID ============\n\n");

p := (1,2,3,4); 
q := (1,3); 
d8 := Group(p,q);
SetName( d8, "d8" );
D0 := DomainWithSingleObject( d8, 0 ); 
Print( "D0 = ", D0, "\n" ); 
Print( KnownTruePropertiesOfObject( D0 ), "\n\n" ); 

p0 := MultiplicativeElementWithObjects( D0, p, 0, 0 ); 
q0 := MultiplicativeElementWithObjects( D0, q, 0, 0 ); 
Print( "[p0,q0,p0*q0] = ", [p0,q0,p0*q0], "\n\n" );


Print("\n=============  MORE THAN ONE CONSTITUENT  ===================\n\n"); 

U1 := UnionOfPieces( [ M78, S123 ] ); 
Print( "U1 = ", U1, "\n" ); 
U2 := UnionOfPieces( [ M456, D0 ] );  
Print( "U2 = ", U2, "\n" ); 
U3 := UnionOfPieces( [ U1, U2] );  
Print( "U3 = ", U3, "\n" ); 

## this should fail since object sets are not disjoint: 
U4 := UnionOfPieces( [ S123, DomainWithSingleObject(d8,-2) ] );  
Print( "U4 = fail? ", U4=fail, "\n\n" ); 
